@@ -1,7 +1,7 @@
 /* 
 
 	Cadabra: a field-theory motivated computer algebra system.
-	Copyright (C) 2001-2009  Kasper Peeters <kasper.peeters@aei.mpg.de>
+	Copyright (C) 2001-2011  Kasper Peeters <kasper.peeters@aei.mpg.de>
 
    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -458,10 +458,11 @@ bool algorithm::apply_recursive(exptree::iterator& st, bool check_cons, int act_
 				count++;
 				++number_of_calls;
 				std::string www=*wit->name;
+//				txtout << "applying at " << *start->name << std::endl;
 				result_t res=apply(start);
 //				debugout << "after apply: " << *(start->multiplier) << std::endl;
 //				exptree::print_recursive_treeform(debugout, start);
-//				exptree::print_recursive_treeform(debugout, tr.begin());
+//				exptree::print_recursive_treeform(txtout, tr.begin());
 				wit=start; // this copying back and forth is needed because wit has different type
 				switch(res) {
 					case l_no_action:
@@ -491,11 +492,11 @@ bool algorithm::apply_recursive(exptree::iterator& st, bool check_cons, int act_
 								}
 							else if(act_at_level!=-1) wit=nextone; // do not follow post_order sequence
 							}
-						else if(wit!=tr.end() && wit->is_identity()) {
+						else if(wit!=tr.end() && wit->is_rational()) { // handle multipliers in products and identity powers
 							bool tryprod=false;
 							bool ispow=true;
 							sibling_iterator tmpact=wit;
-							if( *tr.parent(tmpact)->name=="\\pow" ) {
+							if( *tr.parent(tmpact)->name=="\\pow" && wit->is_identity() ) {
 								iterator par=tr.parent(tmpact);
 								if( tmpact==tr.begin(par) ) { // 1**x = 1
 									node_one(par);
@@ -513,6 +514,7 @@ bool algorithm::apply_recursive(exptree::iterator& st, bool check_cons, int act_
 							
 							if( (!ispow || tryprod) && *tr.parent(tmpact)->name=="\\prod") {
 								iterator tmp=tr.parent(tmpact);
+								multiply(tmp->multiplier, *tmpact->multiplier);
 								tr.erase(tmpact); // may leave us with 0 or 1 children
 								cleanup_anomalous_products(tr,tmp);
 								if(tryprod) wit=tmp;
@@ -523,10 +525,12 @@ bool algorithm::apply_recursive(exptree::iterator& st, bool check_cons, int act_
 								else        wit=nextone;
 								}
 							}
-						
 						else {
-							if(wit!=tr.end()) 
+							if(wit!=tr.end()) {
+//								txtout << "THEN HERE" << std::endl;
 								pushup_multiplier(wit); // Ensure a valid tree wrt. multipliers.
+//								txtout << "THEN HERE DONE" << std::endl;
+								}
 							wit=nextone;
 							}
 						break;
@@ -567,6 +571,9 @@ bool algorithm::apply_recursive(exptree::iterator& st, bool check_cons, int act_
 	//	txtout << "algorithm " << (this_command==tr.end()?"?":*this_command->name) << " worked " << worked 
 	//			 << " failed " << failed << std::endl;
 
+//	tr.debug_verify_consistency();
+//	exptree::print_recursive_treeform(txtout, tr.begin());
+
 	return atleastoneglobal;
 	}
 
@@ -688,22 +695,35 @@ void algorithm::pushup_multiplier(iterator it)
 	if(!tr.is_valid(it)) return;
 	if(*it->multiplier!=1) {
 		if(*it->name=="\\sum") {
+//			txtout << "SUM" << std::endl;
 			sibling_iterator sib=tr.begin(it);
 			while(sib!=tr.end(it)) {
 				multiply(sib->multiplier, *it->multiplier);
+//				txtout << "going up" << std::endl;
+				pushup_multiplier(tr.parent(it));
+//				txtout << "back and back up" << std::endl;
 				pushup_multiplier(sib);
+//				txtout << "back" << std::endl;
 				++sib;
 				}
 			::one(it->multiplier);
 			}
 		else {
+//			txtout << "PUSHUP: " << *it->name << std::endl;
 			if(tr.is_valid(tr.parent(it))) {
+//				txtout << "test propinherit" << std::endl;
+				iterator tmp=tr.parent(it);
+				// tmp not always valid?!? This one crashes hard with a loop!?!
+//				txtout << " of " << *tmp->name << std::endl;
 				const PropertyInherit *pin=properties::get<PropertyInherit>(tr.parent(it));
 				if(pin || *(tr.parent(it)->name)=="\\prod") {
 					multiply(tr.parent(it)->multiplier, *it->multiplier);
+//					txtout << "going up" << std::endl;
 					pushup_multiplier(tr.parent(it));
+//					txtout << "back" << std::endl;
 					::one(it->multiplier);
 					}
+//				else txtout << "not relevant" << std::endl;
 				}
 			}
 		}
@@ -944,7 +964,7 @@ bool algorithm::rename_replacement_dummies(iterator two, bool still_inside_algo)
 //				 << " (index appears " << must_be_empty.count((*it).first) 
 //				 << " times); renaming..." << std::endl;
 		exptree the_key=(*it).first;
-		const Indices *dums=properties::get<Indices>(it->second);
+		const Indices *dums=properties::get<Indices>(it->second, true);
 		if(!dums)
 			throw consistency_error("Failed to find dummy property for $"+*it->second->name+"$ while renaming dummies.");
 //			txtout << "failed to find dummy property for " << *it->second->name << std::endl;
@@ -972,7 +992,7 @@ bool algorithm::rename_replacement_dummies(iterator two, bool still_inside_algo)
 //				 << " (index appears " << must_be_empty.count((*it).first) 
 //				 << " times); renaming..." << std::endl;
 		exptree the_key=(*it).first;
-		const Indices *dums=properties::get<Indices>(it->second);
+		const Indices *dums=properties::get<Indices>(it->second, true);
 		if(!dums)
 			 throw consistency_error("Failed to find dummy property for $"+*it->second->name+"$ while renaming dummies.");
 		assert(dums);
@@ -995,7 +1015,7 @@ bool algorithm::rename_replacement_dummies(iterator two, bool still_inside_algo)
 //				 << " (index appears " << must_be_empty.count((*it).first) 
 //				 << " times); renaming..." << std::endl;
 		exptree the_key=(*it).first;
-		const Indices *dums=properties::get<Indices>(it->second);
+		const Indices *dums=properties::get<Indices>(it->second, true);
 		if(!dums)
 			 throw consistency_error("Failed to find dummy property for $"+*it->second->name+"$ while renaming dummies.");
 		assert(dums);
@@ -1071,7 +1091,7 @@ exptree algorithm::get_dummy(const list_property *dums,
 	while(pr.first!=pr.second) {
 //		txtout << "trying " << std::endl;
 //		tr.print_recursive_treeform(txtout, (*pr.first).second->obj.begin());
-		if(pr.first->second->obj.begin()->is_range_wildcard()) {
+		if(pr.first->second->obj.begin()->is_autodeclare_wildcard()) {
 			std::string base=*pr.first->second->obj.begin()->name_only();
 			int used=max_numbered_name(base, one, two, three, four, five);
 			std::ostringstream str;
@@ -1197,8 +1217,8 @@ void algorithm::determine_intersection(index_map_t& one, index_map_t& two, index
 	{
 	index_map_t::iterator it1=one.begin();
 	while(it1!=one.end()) {
-		const Coordinate *cdn=properties::get<Coordinate>(it1->second);
-		const Symbol     *smb=properties::get<Symbol>(it1->second);
+		const Coordinate *cdn=properties::get<Coordinate>(it1->second, true);
+		const Symbol     *smb=Symbol::get(it1->second, true);
 		if(it1->second->is_integer()==false && !cdn && !smb) {
 			bool move_this_one=false;
 			index_map_t::iterator it2=two.begin();
@@ -1248,8 +1268,8 @@ void algorithm::classify_add_index(iterator it, index_map_t& ind_free, index_map
 	{
 	if((it->fl.parent_rel==str_node::p_sub || it->fl.parent_rel==str_node::p_super) &&
 		it->fl.bracket==str_node::b_none /* && it->is_integer()==false */) {
-		 const Coordinate *cdn=properties::get<Coordinate>(it);
-		 const Symbol     *smb=properties::get<Symbol>(it);
+		const Coordinate *cdn=properties::get<Coordinate>(it, true);
+		const Symbol     *smb=Symbol::get(it, true);
 		 if(it->is_integer() || cdn || smb)
 			  ind_free.insert(index_map_t::value_type(exptree(it), it));
 		 else {
@@ -1397,8 +1417,8 @@ void algorithm::classify_indices(iterator it, index_map_t& ind_free, index_map_t
 				if(!is_first_term) {
 					index_map_t::iterator fri=first_free.begin();
 					while(fri!=first_free.end()) {
-						const Coordinate *cdn=properties::get<Coordinate>(fri->second);
-						const Symbol     *smb=properties::get<Symbol>(fri->second);
+						const Coordinate *cdn=properties::get_composite<Coordinate>(fri->second, true);
+						const Symbol     *smb=Symbol::get(fri->second, true);
                   // integer, coordinate or symbol indices always ok
 						if(fri->second->is_integer()==false && !cdn && !smb) { 
 							if(term_free.count((*fri).first)==0) {
@@ -1417,8 +1437,8 @@ void algorithm::classify_indices(iterator it, index_map_t& ind_free, index_map_t
 						}
 					fri=term_free.begin();
 					while(fri!=term_free.end()) {
-						const Coordinate *cdn=properties::get<Coordinate>(fri->second);
-						const Symbol     *smb=properties::get<Symbol>(fri->second);
+						const Coordinate *cdn=properties::get_composite<Coordinate>(fri->second, true);
+						const Symbol     *smb=Symbol::get(fri->second, true);
                   // integer, coordinate or symbol indices always ok
 						if(fri->second->is_integer()==false && !cdn && !smb) { 
 							if(first_free.count((*fri).first)==0) {
@@ -1520,8 +1540,8 @@ void algorithm::classify_indices(iterator it, index_map_t& ind_free, index_map_t
 			if((sit->fl.parent_rel==str_node::p_sub || sit->fl.parent_rel==str_node::p_super) &&
 				sit->fl.bracket==str_node::b_none /* && sit->is_integer()==false */) {
 				if(*sit->name!="??") {
-					const Coordinate *cdn=properties::get<Coordinate>(sit);
-					const Symbol     *smb=properties::get<Symbol>(sit);
+					const Coordinate *cdn=properties::get<Coordinate>(sit, true);
+					const Symbol     *smb=Symbol::get(sit, true);
 					// integer, coordinate or symbol indices always ok
 					if(sit->is_integer() || cdn || smb) {
 						item_free.insert(index_map_t::value_type(exptree(sit), iterator(sit)));
@@ -1769,6 +1789,7 @@ bool algorithm::cleanup_anomalous_products(exptree& tr, exptree::iterator& it)
 			  tr.begin(it)->multiplier=it->multiplier;
 			  tr.flatten(it);
 			  exptree::iterator tmp=tr.erase(it);
+//			  txtout << "HERRE?" << std::endl;
 			  pushup_multiplier(tmp);
 			  it=tmp;
 			  return true;
