@@ -1,7 +1,7 @@
 /* 
 
 	Cadabra: a field-theory motivated computer algebra system.
-	Copyright (C) 2001-2010  Kasper Peeters <kasper.peeters@aei.mpg.de>
+	Copyright (C) 2001-2011  Kasper Peeters <kasper.peeters@aei.mpg.de>
 
    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -1150,6 +1150,10 @@ void node_printer::print_multiplier(std::ostream& str, exptree::iterator it)
 				str << "(\\frac{" << it->multiplier->get_num() << "}{" 
 					 << it->multiplier->get_den() << "})";
 				}
+			else if(*it->multiplier==-1) {
+				str << "-";
+				turned_one=true;
+				}
 			else {
 				str << "(" << *it->multiplier << ")";
 				}
