@@ -1,7 +1,7 @@
 /* 
 
 	Cadabra: a field-theory motivated computer algebra system.
-	Copyright (C) 2001-2010  Kasper Peeters <kasper.peeters@aei.mpg.de>
+	Copyright (C) 2001-2011  Kasper Peeters <kasper.peeters@aei.mpg.de>
 
    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -358,7 +358,7 @@ void manipulator::output_comment(const std::string& comment) const
 
 // Reads and processes input from streamstack.top() until end of stream.
 // Will close and delete the stream if it derives from std::ifstream. 
-bool manipulator::handle_input()
+void manipulator::handle_input()
 	{
 //	long number_of_lines=0;
 	std::string   oneline;
@@ -378,7 +378,7 @@ bool manipulator::handle_input()
 	while(getline_precut(*(streamstack.top()),oneline)) {
 		replace_cmdline_args(oneline);
 		if(loginput)
-			debugout << oneline << std::endl;
+			txtout << oneline << std::endl;
 		if(oneline.substr(0,10)=="#cellstart" || oneline.substr(0,8)=="#cellend") {
 			txtout << "\n" << oneline << "\n";
 			}
@@ -586,7 +586,10 @@ bool manipulator::handle_input()
 					 // We need to walk back up to the top of the subtree for this 
 					 // expression.
 					 pit = expressions.named_parent(pit, "\\expression");
+//					 txtout << "DONE NOW?" << std::endl;
 //					 exptree::print_recursive_treeform(txtout, pit);
+					 // At this stage, expressions can still have \prod{A}{3} in them, they are
+					 // cleaned up only later?
 					 pit=apply_post_default_rules_(pit);
 					extract_properties_(pit);
 					last_used_equation_number=expressions.equation_number(pit);
@@ -720,7 +723,6 @@ bool manipulator::handle_input()
 	jumpout:
 	cleanup_stream();
 	if(streamstack.size()>0) goto reinput;
-	return true;
 	}
 
 bool manipulator::receive_command(modglue::ipipe& ip)
@@ -791,6 +793,11 @@ void manipulator::cleanup_new_expression_(exptree::iterator it)
    // the last algorithm; check consistency too!
 	// Does not do the right job yet, should be called _after_ the
 	// algorithms have been worked out.
+
+	// This one has to come after generate_indexbracket, otherwise
+	// arguments of those brackets get merged too quickly.
+	index_object_cleanup ioc(expressions, expressions.end());
+	ioc.apply_recursive(it, false);
 	}
 
 exptree::iterator manipulator::apply_pre_default_rules_(exptree::iterator it)
