@@ -1,7 +1,7 @@
 /* 
 
 	Cadabra: a field-theory motivated computer algebra system.
-	Copyright (C) 2001-2010  Kasper Peeters <kasper.peeters@aei.mpg.de>
+	Copyright (C) 2001-2011  Kasper Peeters <kasper.peeters@aei.mpg.de>
 
    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -102,6 +102,7 @@ substitute::substitute(exptree& tr, iterator it)
 			}
 		catch(std::exception& er) {
 			txtout << "substitute: index error in replacement rule " << i+1 << "." << std::endl;
+			txtout << er.what() << std::endl;
 			throw constructor_error();
 			}
 		}
@@ -158,7 +159,6 @@ bool substitute::can_apply(iterator st)
 
 algorithm::result_t substitute::apply(iterator& st)
 	{
-//	txtout << "hi " << *st->name << std::endl;
 //	prod_wrap_single_term(st);
 
    sibling_iterator arrow=tr.arg(args_begin(), use_rule);
@@ -223,9 +223,12 @@ algorithm::result_t substitute::apply(iterator& st)
 			 }
 
 		if(loc!=comparator.replacement_map.end()) { // name wildcards
-//		std::cerr << "rule : " << *((*loc).first.begin()->name) << " -> " 
-//					 << *((*loc).second.begin()->name) << std::endl;
-//		std::cerr << "going to replace " << *it->name << " with " << *((*loc).second.begin()->name) << std::endl;
+//			if((*loc).first.begin()->fl.parent_rel==str_node::p_sub)
+//				std::cerr << "_";
+//			std::cerr << "rule : " << *((*loc).first.begin()->name) << " -> " 
+//						 << *((*loc).second.begin()->name) << std::endl;
+//			std::cerr << it->fl.parent_rel << " ";
+//			std::cerr << "going to replace " << *it->name << " with " << *((*loc).second.begin()->name) << std::endl;
 
 			// When a replacement is made here, and the index is actually
 			// a dummy in the replacement, we screw up the ind_dummy
@@ -245,8 +248,14 @@ algorithm::result_t substitute::apply(iterator& st)
 				it->multiplier=(*loc).second.begin()->multiplier;
 				it->fl=(*loc).second.begin()->fl;
 				}
-			else
-				 it=tr.replace_index(it, (*loc).second.begin());
+			else {
+				// Careful with the multiplier: the object has been matched to the pattern
+				// without taking into account the top-level multiplier. So keep the multiplier
+				// of the thing we are replacing.
+				multiplier_t mt=*it->multiplier;
+				it=tr.replace_index(it, (*loc).second.begin());
+				multiply(it->multiplier, mt);
+				}
 			it->fl.bracket=remember_br;
 			if(rhs_contains_dummies[use_rule])
 				ind_forced.insert(index_map_t::value_type(exptree(it), it));
@@ -259,6 +268,7 @@ algorithm::result_t substitute::apply(iterator& st)
 			multiplier_t tmpmult=*it->multiplier; // remember target multiplier
 			iterator tmp= tr.insert_subtree(it, (*sloc).second);
 			tmp->fl.bracket=it->fl.bracket;
+			tmp->fl.parent_rel=it->fl.parent_rel; // ok?
 			it=tr.erase(it);
 			multiply(tmp->multiplier, tmpmult);
 			subtree_insertion_points.push_back(tmp);
@@ -287,7 +297,7 @@ algorithm::result_t substitute::apply(iterator& st)
 //		txtout << must_be_empty.size() << " dummies have to be relabelled" << std::endl;
 		while(indit!=must_be_empty.end()) {
 			exptree the_key=indit->first;
-			const Indices *dums=properties::get<Indices>(indit->second);
+			const Indices *dums=properties::get<Indices>(indit->second, true);
 			if(dums==0)
 				throw consistency_error("Need to know an index set for " + *indit->second->name +".");
 			exptree relabel=get_dummy(dums, &ind_dummy, &ind_forced, &added_dummies);
@@ -369,14 +379,26 @@ algorithm::result_t substitute::apply(iterator& st)
 	// Cleanup nests on all insertion points and on the top node.
 	for(unsigned int i=0; i<subtree_insertion_points.size(); ++i) {
 		iterator ip=subtree_insertion_points[i];
+		if(*ip->name=="\\sum") { // FIXME: is also in algorithm.cc, and should be factored out
+			if(*ip->multiplier!=1) {
+				sibling_iterator sib=tr.begin(ip);
+				while(sib!=tr.end(ip)) {
+					multiply(sib->multiplier, *ip->multiplier);
+					++sib;
+					}
+				::one(ip->multiplier);
+				}
+			}
 		cleanup_nests(tr, ip);
 		}
 
+//	tr.print_recursive_treeform(txtout, st);
 	
 //	prod_unwrap_single_term(st);
 
 	cleanup_nests(tr, st);
 
+//	tr.print_recursive_treeform(txtout, tr.begin());
 //	prodcollectnum pc(tr, tr.end());
 //	pc.apply(st);
 	tmr.stop();
@@ -431,6 +453,7 @@ bool vary::can_apply(iterator it)
 	{
 	if(*it->name=="\\prod") return true;
 	if(*it->name=="\\sum") return true;
+	if(*it->name=="\\pow") return true;
 	if(is_single_term(it)) return true;
 	if(is_nonprod_factor_in_prod(it)) return true;
 	const Derivative *der = properties::get<Derivative>(it);
@@ -529,6 +552,31 @@ algorithm::result_t vary::apply(iterator& it)
 		expression_modified=true;
 		return l_applied;
 		}
+
+	if(*it->name=="\\pow") { 
+		// Wrap the power in a \cdb_Derivative and then call @prodrule.
+		it=tr.wrap(it, str_node("\\cdb_Derivative"));
+		prodrule pr(tr, it);
+		pr.can_apply(it);
+		pr.apply(it);
+		// Find the '\cdb_Derivative node again'.
+		sibling_iterator sib=tr.begin(it);
+		while(sib!=tr.end()) {
+			if(*sib->name=="\\cdb_Derivative") {
+				tr.flatten(sib);
+				sib=tr.erase(sib);
+				vary vry(tr, this_command);
+				iterator app=sib;
+				if(vry.can_apply(app)) {
+					vry.apply(app);
+					expression_modified=true;
+					}
+				break;
+				}
+			++sib;
+			}
+//		tr.print_recursive_treeform(txtout, it);
+		}
 	
 	der = properties::get<Derivative>(tr.parent(it));
 	acc = properties::get<Accent>(tr.parent(it));
@@ -803,7 +851,7 @@ algorithm::result_t index_rename::apply(iterator& it)
 		std::pair<index_map_t::iterator, index_map_t::iterator> eq=ind_dummy.equal_range(toren->first);
 		index_map_t::iterator dren=eq.first;
 		while(dren!=eq.second) {
-			const Indices *dums=properties::get<Indices>(dren->first.begin());
+			const Indices *dums=properties::get<Indices>(dren->first.begin(), true);
 			if(dums==0)
 				throw consistency_error("Need to know an index set for " + *dren->first.begin()->name +".");
 			exptree relabel=get_dummy(dums, &ind_dummy, &ind_free, &dummies_added);
