@@ -1,7 +1,7 @@
 /* 
 
 	Cadabra: a field-theory motivated computer algebra system.
-	Copyright (C) 2001-2009  Kasper Peeters <kasper.peeters@aei.mpg.de>
+	Copyright (C) 2001-2011  Kasper Peeters <kasper.peeters@aei.mpg.de>
 
    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -102,6 +102,7 @@ std::ostream& exptree::print_recursive_treeform(std::ostream& str, exptree::iter
 		}
 //	str << "  (" << calc_hash(it) << ")";
 //	str << "  (" << depth(it) << ")";
+	str << "  (" << &(*it) << ")";
 	if(!compact_tree) str << std::endl;
 
 	while(beg!=fin) {
@@ -787,6 +788,13 @@ str_node::str_node(const std::string& nm, bracket_t br, parent_rel_t pr)
 //	fl.mark=0;
 	}
 
+void str_node::flip_parent_rel() 
+	{
+	if(fl.parent_rel==p_super)       fl.parent_rel=p_sub;
+	else if(fl.parent_rel==p_sub)    fl.parent_rel=p_super;
+	else throw std::logic_error("flip_parent_rel called on non-index");
+	}
+
 bool str_node::is_zero() const
 	{
 	if(*multiplier==0) return true;
@@ -895,9 +903,18 @@ bool str_node::is_object_wildcard() const
 
 bool str_node::is_range_wildcard() const
 	{
-	size_t pos=name->find('#');
-	if(pos!=std::string::npos)
-		return true;
+	if(name->size()>0) {
+		if((*name)[0]=='#')
+			return true;
+		}
+	return false;
+	}
+
+bool str_node::is_autodeclare_wildcard() const
+	{
+	if(name->size()>0)
+		if((*name)[name->size()-1]=='#')
+			return true;
 	return false;
 	}
 
@@ -938,7 +955,7 @@ nset_t::iterator str_node::name_only()
 		std::string tmp=(*name).substr(0, name->size()-2);
 		return name_set.insert(tmp).first;
 		}
-	else if(is_range_wildcard()) {
+	else if(is_autodeclare_wildcard()) {
 		size_t pos=name->find('#');
 		std::string tmp=(*name).substr(0, pos);
 		return name_set.insert(tmp).first;
@@ -1033,138 +1050,99 @@ void half(rset_t::iterator& num)
 int subtree_compare(exptree::iterator one, exptree::iterator two, 
 						  int mod_prel, bool checksets, int compare_multiplier, bool literal_wildcards) 
 	{
-	// Compare multiplier
-	if(one->is_index()==false && two->is_index()==false) {
-		 if(compare_multiplier==-2 && !two->is_name_wildcard() && !one->is_name_wildcard())
+	// The logic is to compare successive aspects of the two objects, returning a
+	// no-match code if a difference is found at a particular level, or continuing
+	// further down the line if there still is a match.
+	
+	// Compare multipliers. Skip this step if one of the objects is a rational and the
+	// other not, as in that case we are matching values to symbols.
+	if( one->is_rational()==two->is_rational() ) {
+		if(compare_multiplier==-2 && !two->is_name_wildcard() && !one->is_name_wildcard())
 			if(one->multiplier != two->multiplier) {
 				if(*one->multiplier < *two->multiplier) return 2;
 				else return -2;
 				}
-		 }
-		
+		}
+
 	// First lookup some information about the index sets, if any.
 	// (note: to avoid having properties::get enter here recursively, we
-	// perform this check only when both objects are sub/superscripts).
+	// perform this check only when both objects are sub/superscripts, i.e. is_index()==true).
+	// If one and two are sub/superscript, and sit in the same Indices, we keep mult=1, all
+	// other cases get mult=2. 
+
 	int  mult=1;
 	if(one->is_index() && two->is_index() && one->is_rational() && two->is_rational()) mult=2;
-
 	Indices::position_t position_type=Indices::free;
-	if(one->is_index() && two->is_index() && checksets) {
-		const Indices *ind1=properties::get<Indices>(one);
-		const Indices *ind2=properties::get<Indices>(two);
-		if(ind1!=ind2) { 
-			// It may still be that one set is a subset of the other, i.e that the
-			// parent argument of Indices has been used.
-			mult=2;
-// FIXME: this is required for implicit symmetry patterns on split_index objects
-//			if(ind1!=0 && ind2!=0) 
-//				if(ind1->parent_name==ind2->set_name || ind2->parent_name==ind1->set_name)
-//					mult=1;
+	if(one->is_index() && two->is_index()) {
+		if(checksets) {
+			// Strip off the parent_rel because Indices properties are declared without
+			// those.
+			const Indices *ind1=properties::get<Indices>(one, true);
+			const Indices *ind2=properties::get<Indices>(two, true);
+			if(ind1!=ind2) { 
+				// It may still be that one set is a subset of the other, i.e that the
+				// parent argument of Indices has been used.
+				mult=2;
+				// FIXME: this is required for implicit symmetry patterns on split_index objects
+				//			if(ind1!=0 && ind2!=0) 
+				//				if(ind1->parent_name==ind2->set_name || ind2->parent_name==ind1->set_name)
+				//					mult=1;
+				}
+			if(ind1!=0 && ind1==ind2) 
+				position_type=ind1->position_type;
 			}
-//		else std::cout << *one->name << " and " << *two->name << " in same set" << std::endl;
-		if(ind1!=0 && ind1==ind2) 
-			position_type=ind1->position_type;
 		}
+	else mult=2;
 	
 	// Compare sub/superscript relations.
-	if((mod_prel==-2 && position_type!=Indices::free) && one->is_index()) {
+	if((mod_prel==-2 && position_type!=Indices::free) && one->is_index() && two->is_index() ) {
 		if(one->fl.parent_rel!=two->fl.parent_rel) {
 			if(one->fl.parent_rel==str_node::p_sub) return 2;
 			else return -2;
 			}
 		}
 
-	// Once we hit an index, the comparison at a lower level goes differently. 
-	if(one->is_index() && two->is_index()) {
-//		 std::cerr << "is index" << std::endl;
-		if(literal_wildcards==false && (one->is_object_wildcard() || two->is_object_wildcard())) return 0;
-		bool wildcard=(one->is_name_wildcard() || two->is_name_wildcard()) && (literal_wildcards==false);
-//		if(mod_prel2) {
-//		std::cout << *one->name << " vs " << *two->name << " " << mod_prel << " " 
-//					 << checksets << " " << wildcard << std::endl;
-//		if(mod_prel==-2) {
-//			const Coordinate *cdn1=properties::get<Coordinate>(one);
-//			const Coordinate *cdn2=properties::get<Coordinate>(two);
-//			if(cdn1!=0 || cdn2!=0)
-//				wildcard=false;
-//			}
-
-		if(one->name == two->name || wildcard ) {
-			int numch1=exptree::number_of_children(one);
-			int numch2=exptree::number_of_children(two);
-			if(numch1==numch2) {
-				if(numch1==0) {
-					if(wildcard) return 0;
-					if(*one->multiplier < *two->multiplier)     return mult;
-					else if(*one->multiplier > *two->multiplier) return -mult;
-					else return 0;
-					}
-				
-				// FIXME: this only compares the first child
-//				txtout << "subtree compare " << *(one.begin()->name) << " " << *(two.begin()->name) << std::endl;
-				int ret=subtree_compare(one.begin(), two.begin(), -2, false, true, true);
-//				txtout << "ret= " << ret << std::endl;
-				if(ret>0)      return mult;
-				else if(ret<0) return -mult;
-				else {
-					if(*one->multiplier < *two->multiplier)     return 2;
-					else if(*one->multiplier > *two->multiplier) return -2;
-					else return 0;
-					}
-				}
-			else {
-				// One is an index, but the number of children on one and two does not match,
-				// so the structure is different.
-				if(numch1 < numch2) return 2; //mult;
-				else                return -2; //mult;
-				}
-			}
-		// If the names are not equal but they come from the same index set,
-		// return either 1 or -1.
-		// If the names come from different index sets, return 2 or -2.
-		else if(*one->name < *two->name) return mult;
-		else                            return -mult;
-		}
-	else {
-		// Compare node names.
-		if(one->name!=two->name) { 
-			if(literal_wildcards) {
-				if(*one->name < *two->name) return 2;
-				else return -2;
-				}
-			else {
-				if(one->is_object_wildcard()==false && two->is_object_wildcard()==false) {
-					if(one->is_name_wildcard()==false && two->is_name_wildcard()==false) {
-						if(*one->name < *two->name) return 2;
-						else return -2;
-						}
-					}
-				}
-			}
-		}
- 	
-	// Handle object wildcards
-	if(!literal_wildcards) 
+	// Handle object wildcards and comparison
+	if(!literal_wildcards) {
 		if(one->is_object_wildcard() || two->is_object_wildcard())
 			return 0;
+		}
 
-	// Compare number of children.
-//	std::cout << *one->name << " " << *two->name << std::endl;
-//	iterator range_wildcard;
+	// Handle mismatching node names.
+	if(one->name!=two->name) {
+		if(literal_wildcards) {
+			if(*one->name < *two->name) return mult;
+			else return -mult;
+			}
+
+		if( (one->is_autodeclare_wildcard() && two->is_numbered_symbol()) || (two->is_autodeclare_wildcard() && one->is_numbered_symbol()) ) {
+			if( one->name_only() != two->name_only() ) {
+				if(*one->name < *two->name) return mult;
+				else return -mult;
+				}
+			}
+		else if( one->is_name_wildcard()==false && two->is_name_wildcard()==false ) {
+			if(*one->name < *two->name) return mult;
+			else return -mult;
+			}
+		}
+
+	// Now turn to the child nodes. Before comparing them directly, first compare
+	// the number of children, taking into account range wildcards.
 	int numch1=exptree::number_of_children(one);
 	int numch2=exptree::number_of_children(two);
 
-	if(numch1>0 && one.begin()->is_range_wildcard()) {
+//	if(numch1>0 && one.begin()->is_range_wildcard()) {
 		// FIXME: insert the code from props.cc here, ditto in the next if.
-		return 0;
-		}
-	if(numch2>0 && two.begin()->is_range_wildcard()) return 0;
+//		return 0;
+//		}
+
+//	if(numch2>0 && two.begin()->is_range_wildcard()) return 0;
 
 	if(numch1!=numch2) {
 		if(numch1<numch2) return 2;
 		else return -2;
 		}
-//	txtout << numch1 << " == " << numch2 << std::endl;
 
 	// Compare actual children.
 	exptree::sibling_iterator sib1=one.begin(), sib2=two.begin();
@@ -1177,7 +1155,7 @@ int subtree_compare(exptree::iterator one, exptree::iterator two,
 	while(sib1!=one.end()) {
 		int ret=subtree_compare(sib1,sib2, mod_prel, checksets, compare_multiplier, literal_wildcards);
 		if(abs(ret)>1)
-			return ret;
+			return ret/abs(ret)*mult;
 		if(ret!=0 && remember_ret==0) 
 			remember_ret=ret;
 		++sib1;
@@ -1256,7 +1234,7 @@ bool tree_exact_less_obj::operator()(const exptree& one, const exptree& two) con
 
 bool tree_exact_less_no_wildcards_obj::operator()(const exptree& one, const exptree& two) const
 	{
-	return tree_exact_less(one, two, -2, true, -2, true);
+	return tree_exact_less(one, two, -2, true, 0, true);
 	}
 
 bool tree_exact_less_no_wildcards_mod_prel_obj::operator()(const exptree& one, const exptree& two) const
@@ -1360,18 +1338,27 @@ exptree_comparator::match_t exptree_comparator::compare(const exptree::iterator&
 	else if(one->is_object_wildcard())
 		objectpattern=true;
 	else if(is_index && one->is_integer()==false) {
-		const Coordinate *cdn1=properties::get<Coordinate>(one);
+		const Coordinate *cdn1=properties::get<Coordinate>(one, true);
 		if(cdn1==0)
 			implicit_pattern=true;
 		}
 		
 	if(pattern || (implicit_pattern && two->is_integer()==false)) { 
-//		std::cerr << "three" << std::endl;
 		// The above is to ensure that we never match integers to implicit patterns.
 
-		bool tested_full=true;
+		// We want to search the replacement map for replacement rules which we have
+		// constructed earlier, and discard the current match if it conflicts those 
+		// rules. This is to make sure that e.g. a pattern k1_a k2_a does not match 
+		// an expression k1_c k2_d.
+		// 
+		// In order to ensure that a replacement rule for a lower index is also
+		// triggering a rule for an upper index, we simply store both rules (see
+		// below) so that searching for rules can remain simple.
+
 		replacement_map_t::iterator loc=replacement_map.find(one);
 
+		bool tested_full=true;
+
 		// If this is a pattern with a non-zero number of children, 
 		// also search the pattern without the children.
 		if(loc == replacement_map.end() && exptree::number_of_children(one)!=0) {
@@ -1384,15 +1371,14 @@ exptree_comparator::match_t exptree_comparator::compare(const exptree::iterator&
 		if(loc!=replacement_map.end()) {
 //			std::cerr << "found!" << std::endl;
 			// If this is an index/pattern, try to match the whole index/pattern.
-			// We want to make sure that e.g. a pattern k1_a k2_a does not match an expression k1_c k2_d.
-
 			int cmp;
+
 			if(tested_full) 
-				cmp=subtree_compare((*loc).second.begin(), two, 0 /* KP: do not switch this to -2 (kk.cdb fails) */); 
+				cmp=subtree_compare((*loc).second.begin(), two, -2 /* KP: do not switch this to -2 (kk.cdb fails) */); 
 			else {
 				exptree tmp2(two);
 				tmp2.erase_children(tmp2.begin());
-				cmp=subtree_compare((*loc).second.begin(), tmp2.begin(), 0 /* KP: see above */); 
+				cmp=subtree_compare((*loc).second.begin(), tmp2.begin(), -2 /* KP: see above */); 
 				}
 //			std::cerr << " pattern " << *two->name
 //						 << " should be " << *((*loc).second.begin()->name)  
@@ -1409,8 +1395,9 @@ exptree_comparator::match_t exptree_comparator::compare(const exptree::iterator&
 
 //			std::cerr << "index check " << *one->name << " " << *two->name << std::endl;
 
-			const Indices *t1=properties::get<Indices>(one);
-			const Indices *t2=properties::get<Indices>(two);
+			const Indices *t1=properties::get<Indices>(one, true);
+			const Indices *t2=properties::get<Indices>(two, true);
+//			std::cerr << t1 << " " << t2 << std::endl;
 			if( (t1 || t2) && implicit_pattern ) {
 				if(t1 && t2) {
 					if((*t1).set_name != (*t2).set_name) {
@@ -1424,9 +1411,26 @@ exptree_comparator::match_t exptree_comparator::compare(const exptree::iterator&
 					}
 				}
 			// The index types match, so register this replacement rule.
-//			std::cerr << "registering " << *one->name << " " << *two->name << std::endl;
+//			std::cerr << "registering ";
+//			if(one->fl.parent_rel==str_node::p_super) std::cerr << "^";
+//			if(one->fl.parent_rel==str_node::p_sub)   std::cerr << "_";
+//			std::cerr << *one->name << " ";
+//			if(two->fl.parent_rel==str_node::p_super) std::cerr << "^";
+//			if(two->fl.parent_rel==str_node::p_sub)   std::cerr << "_";
+//			std::cerr << *two->name << std::endl;
+
 			replacement_map[one]=two;
 			
+			// if this is an index, also store the pattern with the parent_rel flipped
+			if(one->is_index()) {
+				exptree cmptree1(one);
+				exptree cmptree2(two);
+				cmptree1.begin()->flip_parent_rel();
+				if(two->is_index())
+					cmptree2.begin()->flip_parent_rel();
+				replacement_map[cmptree1]=cmptree2;
+				}
+			
 			// if this is a pattern and the pattern has a non-zero number of children,
 			// also add the pattern without the children
 			if(exptree::number_of_children(one)!=0) {
@@ -1435,6 +1439,13 @@ exptree_comparator::match_t exptree_comparator::compare(const exptree::iterator&
 				tmp2.erase_children(tmp2.begin());
 				replacement_map[tmp1]=tmp2;
 				}
+			// and if this is a pattern also insert the one without the parent_rel
+			if(one->is_name_wildcard()) {
+				exptree tmp1(one), tmp2(two);
+				tmp1.begin()->fl.parent_rel=str_node::p_none;
+				tmp2.begin()->fl.parent_rel=str_node::p_none;
+				replacement_map[tmp1]=tmp2;
+				}
 			}
 		
 		// Return a match of the appropriate type
@@ -1592,7 +1603,9 @@ bool exptree_ordering::should_swap(exptree::iterator obj, int subtree_comparison
 	int num1, num2;
 	const SortOrder *so1=properties::get_composite<SortOrder>(one,num1);
 	const SortOrder *so2=properties::get_composite<SortOrder>(two,num2);
-	
+
+//	std::cerr << so1 << " " << so2 << " " << subtree_comparison << std::endl;
+
 	if(so1==0 || so2==0) { // No sort order known
 		if(subtree_comparison<0) return true;
 		return false;
@@ -1602,6 +1615,7 @@ bool exptree_ordering::should_swap(exptree::iterator obj, int subtree_comparison
 		return false;
 		}
 	else {
+//		std::cerr << num1 << " " << num2 << std::endl;
 		if(so1==so2) {
 			if(num1>num2) return true;
 			return false;
@@ -1621,14 +1635,15 @@ int exptree_ordering::can_swap_prod_obj(exptree::iterator prod, exptree::iterato
 	int sign=1;
 	exptree::sibling_iterator sib=prod.begin();
 	while(sib!=prod.end()) {
-		const Indices *ind1=properties::get_composite<Indices>(sib);
-		const Indices *ind2=properties::get_composite<Indices>(obj);
+		const Indices *ind1=properties::get_composite<Indices>(sib, true);
+		const Indices *ind2=properties::get_composite<Indices>(obj, true);
 		if(! (ind1!=0 && ind2!=0) ) { // If both objects are actually real indices, 
 			                           // then we do not include their commutativity property
 			                           // in the sign. This is because the routines that use
                                     // can_swap_prod_obj all test for such index-index 
                                     // swaps separately.
-			int es=subtree_compare(sib, obj);
+			int es=subtree_compare(sib, obj, 0);
+//			std::cout << "  " << *sib->name << " " << *obj->name << " " << es << std::endl;
 			sign*=can_swap(sib, obj, es, ignore_implicit_indices);
 			if(sign==0) break;
 			}
@@ -1715,11 +1730,11 @@ int exptree_ordering::can_swap_ilist_ilist(exptree::iterator obj1, exptree::iter
 		exptree::index_iterator it2=exptree::begin_index(obj2);
 		while(it2!=exptree::end_index(obj2)) {
 			// Only deal with real indices here, i.e. those carrying an Indices property.
-			const Indices *ind1=properties::get_composite<Indices>(it1);
-			const Indices *ind2=properties::get_composite<Indices>(it1);
+			const Indices *ind1=properties::get_composite<Indices>(it1, true);
+			const Indices *ind2=properties::get_composite<Indices>(it2, true);
 			if(ind1!=0 && ind2!=0) {
-				const CommutingBehaviour *com1 =properties::get_composite<CommutingBehaviour>(it1);
-				const CommutingBehaviour *com2 =properties::get_composite<CommutingBehaviour>(it2);
+				const CommutingBehaviour *com1 =properties::get_composite<CommutingBehaviour>(it1, true);
+				const CommutingBehaviour *com2 =properties::get_composite<CommutingBehaviour>(it2, true);
 				
 				if(com1!=0  &&  com1 == com2) 
 					sign *= com1->sign();
@@ -1763,7 +1778,8 @@ int exptree_ordering::can_swap(exptree::iterator one, exptree::iterator two, int
 
 		// First of all, check whether there is an explicit declaration for the commutativity 
 		// of these two symbols.
-		const CommutingBehaviour *com = properties::get_composite<CommutingBehaviour>(one, two);
+//		std::cout << *one->name << " explicit " << *two->name << std::endl;
+		const CommutingBehaviour *com = properties::get_composite<CommutingBehaviour>(one, two, true);
 		
 		if(com) {
 //			std::cout << "explicit " << com->sign() << std::endl;
@@ -1786,8 +1802,8 @@ int exptree_ordering::can_swap(exptree::iterator one, exptree::iterator two, int
 		}
 
 	// Do we need to use Self* properties?
-	const SelfCommutingBehaviour *sc1 =properties::get_composite<SelfCommutingBehaviour>(one);
-	const SelfCommutingBehaviour *sc2 =properties::get_composite<SelfCommutingBehaviour>(two);
+	const SelfCommutingBehaviour *sc1 =properties::get_composite<SelfCommutingBehaviour>(one, true);
+	const SelfCommutingBehaviour *sc2 =properties::get_composite<SelfCommutingBehaviour>(two, true);
 	if( (sc1!=0 && sc1==sc2) ) 
 		return sc1->sign();
 
@@ -1828,9 +1844,14 @@ bool exptree_comparator::satisfies_conditions(exptree::iterator conditions, std:
 			exptree::sibling_iterator lhs=cond.begin();
 			exptree::sibling_iterator rhs=lhs;
 			++rhs;
-			// If we have a match, all indices have replacement rules.
+			// Lookup the replacement rules for the two given objects, and return true if 
+			// those rules give a different result. But first check that there are rules
+			// to start with.
+//			std::cerr << *lhs->name  << " !=? " << *rhs->name << std::endl;
+			if(replacement_map.find(exptree(lhs))==replacement_map.end() ||
+				replacement_map.find(exptree(rhs))==replacement_map.end()) return true;
+//			std::cerr << *lhs->name  << " !=?? " << *rhs->name << std::endl;
 			if(tree_exact_equal(replacement_map[exptree(lhs)], replacement_map[exptree(rhs)])) {
-//				txtout << *lhs->name  << " = " << *rhs->name << std::endl;
 				return false;
 				}
 			}
@@ -1911,38 +1932,15 @@ bool exptree_comparator::satisfies_conditions(exptree::iterator conditions, std:
 
 bool exptree_is_equivalent::operator()(const exptree& one, const exptree& two)
 	{
-	exptree_comparator comparator;
-	
-	comparator.lhs_contains_dummies=true;
-	exptree_comparator::match_t ret;
-
-//	std::cerr << "-------" << std::endl;
-//	exptree::print_recursive_treeform(std::cerr, one.begin());
-//	exptree::print_recursive_treeform(std::cerr, two.begin());
-
-	ret=comparator.equal_subtree(one.begin(), two.begin());
-
-//	std::cerr << "==? " << (ret==exptree_comparator::subtree_match) << " " << ret << std::endl;
-
-	if(ret==exptree_comparator::subtree_match) return true;
-	else                                       return false;
+	int ret=subtree_compare(one.begin(), two.begin());
+	if(ret==0) return true;
+	else       return false;
 	}
 
 bool exptree_is_less::operator()(const exptree& one, const exptree& two)
 	{
-	exptree_comparator comparator;
-	
-	comparator.lhs_contains_dummies=true;
-	exptree_comparator::match_t ret;
-
-//	std::cerr << "-------" << std::endl;
-//	exptree::print_recursive_treeform(std::cerr, one.begin());
-//	exptree::print_recursive_treeform(std::cerr, two.begin());
-
-	ret=comparator.equal_subtree(one.begin(), two.begin());
-
-//	std::cerr << "<? " << (ret==exptree_comparator::no_match_less) << " " << ret << std::endl;
-
-	if(ret==exptree_comparator::no_match_less) return true;
-	else                                       return false;
+	int ret=subtree_compare(one.begin(), two.begin());
+	if(ret < 0) return true;
+	else        return false;
 	}
+
